<?php

namespace Ecommerce\Util;

use Exception\CustomException;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\ORM\DateTime;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\Util\Crypt;

class LoginUtilities 
{
    const LOGIN_CRYPT_STRING = '1460_loginManagement';

    protected $directory;
    protected $service;
    protected $utilities;
    
    private $appId;

    public function __construct($service, $utilities) {
        $this->service = $service;
        $this->utilities = $utilities;
        $this->directory = realpath(__DIR__ . '/Template');
        $this->appId = "PORTAL E-COMMERCE";        
	}
    
    public static function getLoginConfig() {
        return json_decode(file_get_contents(__DIR__ . '/../../../config/config.json'), true);
    }
    
    public static function getLoginIntegration() {
        return json_decode(file_get_contents(__DIR__ . '/../../../config/integration.json'), true)[0];
    }
    
    /**
     * @param $token
     * @param $requestType
     * @param $data
     * @param $dataSetResultPos
     * @param $privateKey
     * @param $url
     * @param bool $keepConnected
     * @param bool $isThrowNotConnectedErrors
     * @return array
     * @throws \Exception
     */
    public function callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url, $keepConnected = false, $isThrowNotConnectedErrors = false)
    {
        set_time_limit(180);
        $notConnectCUrlErrorCodes = array(
            7,  //Failed to connect to host or proxy
            28, //Operation timeout
            35, //SSL connection error
            89, //No connection available
        );
        $header = array();
        $header[] = 'OAuth-Token: ' . $token;
        $header[] = 'OAuth-SecretId: ' . $privateKey;
        if($keepConnected) {
            $header[] = 'OAuth-KeepConnected: ' . $keepConnected;
        }
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: ' . strlen(json_encode($data));

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_URL, $url . $requestType);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);

        $resultCurl = curl_exec($ch);
        $result     = json_decode($resultCurl, true);
        $error      = curl_error($ch);
        $errorCode  = curl_errno($ch);
        curl_close($ch);
        
        if (!empty($result['error'])) {
            $errorCode = isset($result['errorCode']) ? $result['errorCode'] : $errorCode;
            throw new \Exception($result['error'], $errorCode);
        } else if (!empty($error)) {
            if(in_array($errorCode, $notConnectCUrlErrorCodes) && !$isThrowNotConnectedErrors) {
                return null;
            }
            throw new \Exception($error, $errorCode);
        } else if(is_null($result) && $isThrowNotConnectedErrors) {
            throw new \Exception($resultCurl, $errorCode);
        } else {
            return isset($result['dataset'][$dataSetResultPos]) ? $result['dataset'][$dataSetResultPos] : null;
        }
    }
    
    public function getValueInFilterDataSourceByName(array &$filters,$nameFilter,$remove = false)
    {
        foreach ($filters as $key => $filter) {
            $name = $filter['columnName'];
            $values = $filter['value'];
            
            if ($name == $nameFilter ) {
                if ($remove) {
                    unset($filters[$key]);
                }
                
                return $values;
            }
        }
        
        return null;
    }
    
    public function generateRandomToken($length)
    {
        $characters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890';
        $charactersLength = strlen($characters);
        $randomPassword = '';
        
        for ($i = 1; $i <= $length; $i++) {
            $rand = mt_rand(1, $charactersLength);
            $randomPassword .= $characters[$rand-1];
        }
        
        return $randomPassword;
    }
    
    public function zeedhiEncrypt($text, $key = 'login') {
        return Crypt::encrypt($text, $key );
    }

    public function zeedhiDecrypt($text, $key = 'login') {
        return Crypt::decrypt($text, $key );
    }
    
    /**
     * Capitalize the words of the string sent.
     * 
     * @param string $word String that will be capitalized.  
     *
     * @return string
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function capitalizeWords($word)
    {
        return ucwords(strtolower($word));
    }
    
    /**
     * @param CustomException $e
     * @return Error
     */
    public function getErrorObjectFromException(CustomException $e) 
    {
        $error = array (
            'CODE'   => $e->getMessage(),
            'PARAMS' => $e->getParams()
        );
        
        return new Error(json_encode($error), $e->getCode(), $e->getTraceAsString());
    }

    public function getTemplate($file, $search_replace = array())
    {
        try {
            if ($file["name"]) {
                $path = $this->getFilePath($file);
                $content = $this->getFileContents($path);

                if (!empty($search_replace)) {
                    foreach ($search_replace as $index => $value) {
                        $search[] = $index;
                        $replace[] = $value;
                    }
                    
                    $content = $this->replaceTags($search, $replace, $content);
                }
                
                return $content;
            } else {
                throw new \Exception("Não foi enviado o nome do arquivo.", 400);
            }
        } catch (Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }

    private function getFilePath($file)
    {
        $fileName = $file["name"];
        $fileExtension = isset($file["extension"]) ? $file["extension"] : ".html";
        $filePath = $this->directory . '/' . $fileName . $fileExtension;
        
        if (file_exists($filePath)) {
            return $filePath;
        } else {
            throw new \Exception("Caminho de arquivo não encontrado.", 400);
        }
    }
    
    private function getFileContents($path)
    {
        if (file_exists($path)) {
            $content = file_get_contents($path);
            return mb_convert_encoding($content, 'UTF-8', mb_detect_encoding($content, 'UTF-8, ISO-8859-1', true));
        } else {
            throw new \Exception("Caminho de arquivo não encontrado.", 400);
        }
    }
    
    private function replaceTags($search, $replace, $content)
    {
        $content = str_replace($search, $replace, $content, $count);

        if ($count > 0) {
            return $content;
        } else {
            throw new \Exception("Nenhuma alteração das tags foi feita. Verifique os arrays enviados.", 400);
        }
    }

    public function sendRequestNewPasswordMail($subject, $content, $emails)
    {
        try {
            $unitId = $this->utilities->getFilialParasupri();
            $logo = $this->utilities->getLogo($this->appId);
            $lgpd = $this->utilities->getLgpd($unitId);
            
            $imgAttr = "width='200px'";
            $this->utilities->getLogoAttributes($logo, $imgAttr);
            
            $contentHeader = "<p class='bold' style='text-align: center; color: #ED2424'>" . $subject . "</p>"; 
            
            $file = array("name" => "email", "extension" => ".html");
            $replace = array(
                "%ATTRIMG%" => $imgAttr, 
                "%LOGOSRC%" => $logo, 
                "%SUBJECT%" => $contentHeader, 
                "%MESSAGE%" => $content, 
                "%LGPD%" => $lgpd
            );
            $content = $this->getTemplate($file, $replace);
            $smtpMail = InstanceManager::getInstance()->getService('smtpUtil');            
            
            $smtpMail->setSubject($subject);
            $smtpMail->sendMail($content, $emails);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
}